package com.qmth.ops.api.controller.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.api.security.Permission;
import com.qmth.ops.api.vo.AppVersionVO;
import com.qmth.ops.biz.domain.App;
import com.qmth.ops.biz.query.AppQuery;
import com.qmth.ops.biz.service.AppService;
import com.qmth.ops.biz.service.VersionService;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/app")
public class AppController {

    @Resource
    private AppService appService;

    @Resource
    private VersionService versionService;

    @PostMapping("/query")
    public IPage<AppVersionVO> query(AppQuery query) {
        return appService.query(query).convert(app -> new AppVersionVO(app, versionService));
    }

    @PostMapping("/list")
    public List<AppVersionVO> list(AppQuery query) {
        return appService.list(query).stream().map(app -> new AppVersionVO(app, versionService))
                .collect(Collectors.toList());
    }

    @PostMapping("/insert")
    public AppVersionVO insert(@RequestAttribute AdminSession adminSession, App app) {
        adminSession.hasPermission(Permission.APP_INSERT);
        appService.insert(app);
        return new AppVersionVO(appService.getById(app.getId()), versionService);
    }

    @PostMapping("/update")
    public AppVersionVO update(@RequestAttribute AdminSession adminSession, App app) {
        adminSession.hasPermission(Permission.APP_EDIT);
        appService.update(app);
        return new AppVersionVO(appService.getById(app.getId()), versionService);
    }

}

