package com.qmth.ops.api.controller.admin;

import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.api.security.Permission;
import com.qmth.ops.api.vo.LlmSupplierVO;
import com.qmth.ops.biz.domain.LlmSupplier;
import com.qmth.ops.biz.service.LlmClientService;
import com.qmth.ops.biz.service.LlmSupplierService;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/llm/supplier")
public class LlmSupplierController {

    @Resource
    private LlmSupplierService llmSupplierService;

    @Resource
    private LlmClientService llmClientService;

    @PostMapping("/list")
    public List<LlmSupplierVO> list() {
        return llmSupplierService.list().stream().map(LlmSupplierVO::new).collect(Collectors.toList());
    }

    @PostMapping("/detail")
    public LlmSupplier detail(@RequestParam Long id) {
        return llmSupplierService.getById(id);
    }

    @PostMapping("/insert")
    public LlmSupplier insert(@RequestAttribute AdminSession adminSession, LlmSupplier supplier) {
        adminSession.hasPermission(Permission.LLM_SUPPLIER_INSERT);
        return llmSupplierService.insert(supplier);
    }

    @PostMapping("/update")
    public LlmSupplier update(@RequestAttribute AdminSession adminSession, LlmSupplier supplier) {
        adminSession.hasPermission(Permission.LLM_SUPPLIER_EDIT);
        llmSupplierService.update(supplier);
        llmClientService.updateBySupplier(supplier);
        return llmSupplierService.getById(supplier.getId());
    }

}
