package com.qmth.ops.api.controller.admin;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.api.security.Permission;
import com.qmth.ops.api.vo.ModuleTypeVO;
import com.qmth.ops.biz.domain.Module;
import com.qmth.ops.biz.domain.ModuleType;
import com.qmth.ops.biz.query.ModuleQuery;
import com.qmth.ops.biz.service.ModuleService;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/module")
public class ModuleController {

    @Resource
    private ModuleService moduleService;

    @RequestMapping("/types")
    @Aac(auth = false)
    public Object types() {
        return Arrays.stream(ModuleType.values()).map(ModuleTypeVO::new).toArray();
    }

    @PostMapping("/insert")
    public Module insert(@RequestAttribute AdminSession adminSession, Module module) {
        adminSession.hasPermission(Permission.MODULE_INSERT);
        return moduleService.insert(module);
    }

    @PostMapping("/update")
    public Module update(@RequestAttribute AdminSession adminSession, Module module) {
        adminSession.hasPermission(Permission.MODULE_EDIT);
        return moduleService.update(module);
    }

    @PostMapping("/list")
    public List<Module> list(ModuleQuery query) {
        return moduleService.list(query);
    }

}
