package com.qmth.ops.api.controller.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.core.solar.enums.OrgType;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.api.security.Permission;
import com.qmth.ops.api.vo.OrgTypeVO;
import com.qmth.ops.api.vo.OrgVO;
import com.qmth.ops.api.vo.SuccessVO;
import com.qmth.ops.biz.domain.Org;
import com.qmth.ops.biz.query.OrgQuery;
import com.qmth.ops.biz.service.FileService;
import com.qmth.ops.biz.service.OrgService;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.Arrays;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/org")
public class OrgController {

    @Resource
    private OrgService orgService;

    @Resource
    private FileService fileService;

    @RequestMapping("/types")
    @Aac(auth = false)
    public Object getTypes() {
        return Arrays.stream(OrgType.values()).map(OrgTypeVO::new).toArray();
    }

    @PostMapping("/query")
    public IPage<OrgVO> query(OrgQuery query) {
        return orgService.findByQuery(query).convert(OrgVO::new);
    }

    @PostMapping("/insert")
    public OrgVO insert(@RequestAttribute AdminSession adminSession, Org org,
            @RequestParam(required = false) MultipartFile logoFile, @RequestParam(required = false) String logoMd5)
            throws Exception {
        adminSession.hasPermission(Permission.ORG_INSERT);
        org = orgService.insert(org);
        if (logoFile != null && logoMd5 != null) {
            fileService.uploadOrgLogo(org, logoFile, logoMd5);
            org = orgService.update(org);
        }
        return new OrgVO(org);
    }

    @PostMapping("/update")
    public OrgVO update(@RequestAttribute AdminSession adminSession, Org org,
            @RequestParam(required = false) MultipartFile logoFile, @RequestParam(required = false) String logoMd5)
            throws Exception {
        adminSession.hasPermission(Permission.ORG_EDIT);
        org = orgService.update(org);
        if (logoFile != null && logoMd5 != null) {
            fileService.uploadOrgLogo(org, logoFile, logoMd5);
            org = orgService.update(org);
        }
        return new OrgVO(org);
    }

    @PostMapping("/toggle")
    public Object toggle(@RequestAttribute AdminSession adminSession, @RequestParam Long id,
            @RequestParam Boolean enable) {
        adminSession.hasPermission(Permission.ORG_EDIT);
        return new SuccessVO(orgService.toggle(id, enable));
    }
}
