package com.qmth.ops.api.controller.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.dto.LoginResult;
import com.qmth.ops.api.dto.PermissionDTO;
import com.qmth.ops.api.dto.UserForm;
import com.qmth.ops.api.dto.UserPermissionDTO;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.api.security.Permission;
import com.qmth.ops.api.vo.CodeNameVO;
import com.qmth.ops.api.vo.UpdateTimeVO;
import com.qmth.ops.biz.domain.Role;
import com.qmth.ops.biz.domain.User;
import com.qmth.ops.biz.query.UserQuery;
import com.qmth.ops.biz.service.FileService;
import com.qmth.ops.biz.service.UserPermissionService;
import com.qmth.ops.biz.service.UserService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/user")
public class UserController {

    @Resource
    private UserService userService;

    @Resource
    private UserPermissionService userPermissionService;

    @Resource
    private FileService fileService;

    @PostMapping("/login")
    @Aac(auth = false)
    public LoginResult login(@Validated(UserForm.LoginGroup.class) UserForm form) {
        User user = userService.findByLoginName(form.getLoginName());
        if (user == null) {
            throw new ParameterException("登录名错误");
        }
        if (!user.getEnable()) {
            throw new ParameterException("用户已禁用");
        }
        if (!user.buildPassword(form.getPassword()).equals(user.getPassword())) {
            throw new ParameterException("密码错误");
        }
        return new AdminSession(userService.changeAccessToken(user), userPermissionService)
                .getLoginResult(fileService.getServer());
    }

    @RequestMapping("/roles")
    @Aac(auth = false)
    public Object getRoles() {
        return Arrays.stream(Role.values()).map(role -> new CodeNameVO(role.getCode(), role.getName())).toArray();
    }

    @PostMapping("/query")
    public IPage<User> query(@RequestAttribute AdminSession adminSession, UserQuery query) {
        return userService.query(query);
    }

    @PostMapping("/list")
    public List<User> list(@RequestAttribute AdminSession adminSession, UserQuery query) {
        return userService.list(query);
    }

    @PostMapping("/insert")
    public User insert(@RequestAttribute AdminSession adminSession,
            @Validated(UserForm.InsertGroup.class) UserForm form) {
        adminSession.hasPermission(Permission.USER_INSERT);
        return userService.insert(form.build());
    }

    @PostMapping("/update")
    public User update(@RequestAttribute AdminSession adminSession,
            @Validated(UserForm.UpdateGroup.class) UserForm form) {
        adminSession.hasPermission(Permission.USER_EDIT);
        return userService.update(form.build());
    }

    @PostMapping("/change_password")
    public User changePassword(@RequestAttribute AdminSession adminSession, @RequestParam String password) {
        User user = adminSession.getUser();
        user.setPassword(password);
        return userService.changePassword(user);
    }

    @PostMapping("/permission/query")
    public List<PermissionDTO> permissionQuery(@RequestParam Long userId) {
        return userPermissionService.listByUser(userService.getById(userId)).stream().map(PermissionDTO::new)
                .collect(Collectors.toList());
    }

    @PostMapping("/permission/save")
    public UpdateTimeVO permissionSave(@RequestAttribute AdminSession adminSession,
            @Validated @RequestBody UserPermissionDTO userPermission) {
        adminSession.hasPermission(Permission.USER_EDIT);
        User user = userService.getById(userPermission.getUserId());
        userPermissionService.saveAll(user, userPermission.getUserPermissionList());
        return new UpdateTimeVO().set("userId", user.getId());
    }

    @PostMapping("/permission/delete")
    public UpdateTimeVO permissionDelete(@RequestAttribute AdminSession adminSession,
            @Validated @RequestBody UserPermissionDTO userPermission) {
        adminSession.hasPermission(Permission.USER_EDIT);
        User user = userService.getById(userPermission.getUserId());
        for (PermissionDTO dto : userPermission.getPermissions()) {
            userPermissionService.delete(user, dto.getPermission());
        }
        return new UpdateTimeVO().set("userId", userPermission.getUserId());
    }

}
