package com.qmth.ops.api.controller.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.ops.api.binder.VersionNumberBinder;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.api.security.Permission;
import com.qmth.ops.biz.domain.Version;
import com.qmth.ops.biz.query.VersionQuery;
import com.qmth.ops.biz.service.AppService;
import com.qmth.ops.biz.service.VersionService;
import com.qmth.ops.biz.utils.VersionNumber;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/version")
public class VersionController {

    @Resource
    private VersionService versionService;

    @Resource
    private AppService appService;

    @InitBinder
    public void initBinder(WebDataBinder dataBinder) {
        dataBinder.addCustomFormatter(new VersionNumberBinder());
    }

    @PostMapping("/insert")
    public Version insert(@RequestAttribute AdminSession adminSession, @RequestParam Long appId,
            @RequestParam VersionNumber name) {
        adminSession.hasPermission(Permission.VERSION_INSERT, appId);
        return versionService.insert(appService.getById(appId), name);
    }

    @PostMapping("/update")
    public Version edit(@RequestAttribute AdminSession adminSession, Version version) {
        Version previous = versionService.getById(version.getId());
        adminSession.hasPermission(Permission.VERSION_EDIT, previous.getAppId());
        return versionService.update(version);
    }

    @PostMapping("/query")
    public IPage<Version> query(VersionQuery query) {
        return versionService.query(query);
    }

}
