package com.qmth.ops.api.controller.export;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.UnauthorizedException;
import com.qmth.ops.api.binder.FileFormatBinder;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.biz.domain.*;
import com.qmth.ops.biz.service.*;
import com.qmth.ops.biz.utils.BranchName;
import com.qmth.ops.biz.utils.PropertyFileUtil;
import com.qmth.ops.biz.utils.VersionNumber;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

@RestController
@RequestMapping(OpsApiConstants.EXPORT_URI_PREFIX + "/property")
@Aac(auth = false)
public class PropertyExportController {

    @Resource
    private UserService userService;

    @Resource
    private AppService appService;

    @Resource
    private ModuleService moduleService;

    @Resource
    private EnvService envService;

    @Resource
    private VersionService versionService;

    @Resource
    private PropertyService propertyService;

    @InitBinder
    public void initBinder(WebDataBinder dataBinder) {
        dataBinder.addCustomFormatter(new FileFormatBinder());
    }

    @RequestMapping("/{appCode}/{moduleCode}/{envCode}/{branchName}")
    public void exportPropertyFile(@PathVariable String appCode, @PathVariable String moduleCode,
            @PathVariable String envCode, @PathVariable String branchName, @RequestParam String secret,
            @RequestParam(required = false, defaultValue = "false") Boolean encrypt, HttpServletResponse response)
            throws IOException {
        exportPropertyFile(appCode, moduleCode, envCode, new BranchName(branchName).getVersionNumber(), secret,
                encrypt != null ? encrypt : false, response);
    }

    private void exportPropertyFile(String appCode, String moduleCode, String envCode, VersionNumber versionNumber,
            String exportSecret, boolean encrypt, HttpServletResponse response) throws IOException {
        User user = userService.findByExportSecret(exportSecret);
        if (user == null || !user.hasRole(Role.OPS)) {
            throw new UnauthorizedException("鉴权失败");
        }
        App app = appService.findByCode(appCode);
        if (app == null) {
            throw new ParameterException("app.code不存在");
        }
        Module module = moduleService.findByAppAndCode(app.getId(), moduleCode);
        if (module == null) {
            throw new ParameterException("module.code不存在");
        }
        Env env = envService.findByAppAndCode(app.getId(), envCode);
        if (env == null) {
            throw new ParameterException("env.code不存在");
        }
        Version version = versionService.findByAppAndNumber(app.getId(), versionNumber);
        if (version == null) {
            throw new ParameterException("version不存在");
        }
        List<PropertyItem> list = propertyService.mergePropertyList(appCode, version, module, env.getId(), encrypt);
        response.reset();
        response.setContentType("application/octet-stream");
        response.setHeader("Content-Disposition", "attachment; filename=application.properties");
        PropertyFileUtil.write(list, response.getOutputStream(),
                appCode + "_" + moduleCode + "_" + version.getName() + "@" + envCode);
    }

}
