package com.qmth.ops.biz.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.core.fss.store.FileStore;
import com.qmth.boot.core.fss.utils.FssUtils;
import com.qmth.boot.tools.device.DeviceInfo;
import com.qmth.boot.tools.models.ByteArray;
import com.qmth.ops.biz.domain.Org;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

@Service
public class FileService {

    private static String ORG_LOGO_DIR = FssUtils.buildPath("org", "logo");

    private static String DEVICE_INFO_DIR = FssUtils.buildPath("device");

    private FileStore fileStore;

    public FileService(FileStore fileStore) {
        this.fileStore = fileStore;
    }

    public String getServer() {
        return fileStore.getServer();
    }

    public void uploadOrgLogo(Org org, MultipartFile file, String md5) throws Exception {
        String extension = getExtension(file.getOriginalFilename());
        String path = FssUtils
                .buildPath(ORG_LOGO_DIR, org.getId().toString() + (extension != null ? ("." + extension) : ""));
        fileStore.write(path, file.getInputStream(), md5);
        org.setLogo(path);
    }

    public ByteArray getOrgLogo(Org org) throws Exception {
        if (org.getLogo() != null) {
            InputStream ins = fileStore.read(org.getLogo());
            ByteArray data = ByteArray.fromInputStream(ins);
            ins.close();
            return data;
        } else {
            return ByteArray.fromArray(null);
        }
    }

    public void uploadDeviceInfo(String deviceId, DeviceInfo info) throws Exception {
        byte[] data = new ObjectMapper().writeValueAsBytes(info);
        String path = FssUtils.buildPath(DEVICE_INFO_DIR, deviceId + ".json");
        fileStore.write(path, new ByteArrayInputStream(data), ByteArray.md5(data).toBase64());
    }

    public ByteArray getDeviceInfo(String deviceId) throws Exception {
        String path = FssUtils.buildPath(DEVICE_INFO_DIR, deviceId + ".json");
        InputStream ins = fileStore.read(path);
        ByteArray data = ByteArray.fromInputStream(ins);
        ins.close();
        return data;
    }

    public String getExtension(String filename) {
        int index = filename != null ? filename.lastIndexOf(".") : -1;
        return index > -1 ? StringUtils.trimToNull(filename.substring(index + 1)) : null;
    }
}
