package com.qmth.ops.biz.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.qmth.boot.core.ai.model.llm.LlmAppType;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.ops.biz.dao.LlmPromptTemplateDao;
import com.qmth.ops.biz.domain.LlmPromptTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

@Service
public class LlmPromptTemplateService {

    private static final String CACHE_NAME = "llm_prompt_template";

    @Resource
    private LlmOrgConfigService llmOrgConfigService;

    @Resource
    private LlmPromptTemplateDao llmPromptTemplateDao;

    //@Cacheable(value = CACHE_NAME, key = "#id")
    public LlmPromptTemplate findById(Long id) {
        return llmPromptTemplateDao.selectById(id);
    }

    public List<LlmPromptTemplate> findBySupplierAndAppType(Long supplierId, LlmAppType appType) {
        return llmPromptTemplateDao.selectList(new LambdaQueryWrapper<LlmPromptTemplate>()
                .eq(supplierId != null, LlmPromptTemplate::getSupplierId, supplierId)
                .eq(appType != null, LlmPromptTemplate::getAppType, appType));
    }

    @Transactional
    //@CachePut(value = CACHE_NAME, key = "#template.id", unless = "#template==null")
    public void insert(LlmPromptTemplate template) {
        llmPromptTemplateDao.insert(template);
    }

    @Transactional
    //@CachePut(value = CACHE_NAME, key = "#template.id", unless = "#template==null")
    public void update(LlmPromptTemplate template) {
        llmPromptTemplateDao.update(null,
                new LambdaUpdateWrapper<LlmPromptTemplate>().set(LlmPromptTemplate::getSystem, template.getSystem())
                        .set(LlmPromptTemplate::getUser, template.getUser())
                        .set(LlmPromptTemplate::getRemark, template.getRemark())
                        .eq(LlmPromptTemplate::getId, template.getId()));
    }

    @Transactional
    //@CacheEvict(value = CACHE_NAME, key = "#template.id", unless = "#template==null")
    public void delete(LlmPromptTemplate template) {
        if (template == null) {
            throw new ParameterException("提示词模版ID不存在");
        }
        if (llmOrgConfigService.countByPromptTemplate(template) > 0) {
            throw new ParameterException("提示词模版有关联机构，不能删除");
        }
        llmPromptTemplateDao.deleteById(template.getId());
    }

}

