package com.qmth.ops.biz.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.qmth.ops.biz.dao.NginxConfigDao;
import com.qmth.ops.biz.domain.App;
import com.qmth.ops.biz.domain.Env;
import com.qmth.ops.biz.domain.Module;
import com.qmth.ops.biz.domain.NginxConfig;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;

@Service
public class NginxConfigService {

    private static final long GLOBAL_MODULE_ID = 0L;

    private static final long BASELINE_ENV_ID = 0L;

    @Resource
    private NginxConfigDao nginxConfigDao;

    public NginxConfig find(@NotNull App app, @Nullable Env env, @Nullable Module module) {
        return findOne(app.getId(), env != null ? env.getId() : BASELINE_ENV_ID,
                module != null ? module.getId() : GLOBAL_MODULE_ID);
    }

    @Transactional
    public NginxConfig update(@NotNull App app, @Nullable Env env, @Nullable Module module, @NotNull String content) {
        NginxConfig config = new NginxConfig();
        config.setAppId(app.getId());
        config.setEnvId(env != null ? env.getId() : BASELINE_ENV_ID);
        config.setModuleId(module != null ? module.getId() : GLOBAL_MODULE_ID);
        config.setContent(content);
        config.setCreateTime(System.currentTimeMillis());
        config.setUpdateTime(config.getCreateTime());
        nginxConfigDao.delete(new LambdaQueryWrapper<NginxConfig>().eq(NginxConfig::getAppId, config.getAppId())
                .eq(NginxConfig::getEnvId, config.getEnvId()).eq(NginxConfig::getModuleId, config.getModuleId()));
        nginxConfigDao.insert(config);
        return findOne(config.getAppId(), config.getEnvId(), config.getModuleId());
    }

    private NginxConfig findOne(Long appId, Long envId, Long moduleId) {
        return nginxConfigDao.selectOne(
                new LambdaQueryWrapper<NginxConfig>().eq(NginxConfig::getAppId, appId).eq(NginxConfig::getEnvId, envId)
                        .eq(NginxConfig::getModuleId, moduleId));
    }
}
