package com.qmth.ops.biz.service;

import com.qmth.boot.core.ai.model.ocr.ImageType;
import com.qmth.boot.core.ai.model.ocr.OcrType;
import com.qmth.ops.biz.ai.client.OcrApiClient;
import com.qmth.ops.biz.ai.client.OcrApiConfig;
import com.qmth.ops.biz.ai.exception.OcrClientNotFound;
import com.qmth.ops.biz.domain.OcrSupplier;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class OcrClientService {

    private static final Logger log = LoggerFactory.getLogger(OcrClientService.class);

    private OcrApiClient defaultClient;

    private Map<Long, OcrApiClient> clientMap;

    @Resource
    private OcrSupplierService ocrSupplierService;

    @PostConstruct
    public synchronized void init() {
        defaultClient = null;
        clientMap = new HashMap<>();

        List<OcrSupplier> list = ocrSupplierService.list();
        for (OcrSupplier supplier : list) {
            initApiClient(supplier);
        }
    }

    private void initApiClient(OcrSupplier supplier) {
        try {
            String className = OcrApiClient.class.getName().replace(OcrApiClient.class.getSimpleName(), supplier.getClientClass());
            OcrApiConfig config = new OcrApiConfig(supplier);
            Class<?> clientClass = Class.forName(className);

            Object clientInstance = clientClass.getConstructor(OcrApiConfig.class).newInstance(config);
            if (clientInstance instanceof OcrApiClient) {
                OcrApiClient client = (OcrApiClient) clientInstance;
                clientMap.put(supplier.getId(), client);
                // 取第一个enable=true的为默认客户端
                if (supplier.getEnable() && defaultClient == null) {
                    defaultClient = client;
                }
            }
        } catch (Exception e) {
            log.warn("OcrApiClient init error, supplier:{} class:{}", supplier.getName(), supplier.getClientClass());
        }
    }

    public String forImage(Long id, OcrType type, byte[] imageData, ImageType imageType) throws Exception {
        OcrApiClient client = clientMap.get(id);
        if (client == null) {
            throw new OcrClientNotFound(id);
        }
        return client.forImage(type, imageData, imageType);
    }

    public String forImage(OcrType type, byte[] imageData, ImageType imageType) throws Exception {
        if (defaultClient == null) {
            throw new OcrClientNotFound();
        }
        return defaultClient.forImage(type, imageData, imageType);
    }

}
