package com.qmth.ops.biz.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.ops.biz.dao.UserDao;
import com.qmth.ops.biz.domain.User;
import com.qmth.ops.biz.query.UserQuery;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

@Service
public class UserService extends ServiceImpl<UserDao, User> {

    @Resource
    private UserDao userDao;

    public User findByLoginName(String loginName) {
        return userDao.selectOne(new LambdaQueryWrapper<User>().eq(User::getLoginName, loginName));
    }

    public User findByExportSecret(String exportSecret) {
        return userDao.selectOne(new LambdaQueryWrapper<User>().eq(User::getExportSecret, exportSecret));
    }

    @Transactional
    public User insert(User user) {
        user.setExportSecret(RandomStringUtils.random(16, true, true).toLowerCase());
        user.setCreateTime(System.currentTimeMillis());
        user.setUpdateTime(user.getCreateTime());
        user.setPassword(user.buildPassword(user.getPassword()));
        userDao.insert(user);
        return user;
    }

    @Transactional
    public User update(User user) {
        User previous = userDao.selectById(user.getId());
        if (user.getLoginName() != null) {
            previous.setLoginName(user.getLoginName());
        }
        if (user.getName() != null) {
            previous.setName(user.getName());
        }
        if (user.getRole() != null && user.getRole().length > 0) {
            previous.setRole(user.getRole());
        }
        if (user.getPassword() != null) {
            previous.buildPassword(user.getPassword());
        }
        if (user.getEnable() != null) {
            previous.setEnable(user.getEnable());
        }
        previous.setUpdateTime(System.currentTimeMillis());
        userDao.updateById(previous);
        return previous;
    }

    @Transactional
    public User changePassword(User user) {
        User previous = userDao.selectById(user.getId());
        previous.setPassword(previous.buildPassword(user.getPassword()));
        previous.setUpdateTime(System.currentTimeMillis());
        userDao.updateById(previous);
        return previous;
    }

    @Transactional
    public User changeAccessToken(User user) {
        userDao.update(null,
                new LambdaUpdateWrapper<User>().set(User::getAccessToken, RandomStringUtils.random(16, true, false))
                        .eq(User::getId, user.getId()));
        return userDao.selectById(user.getId());
    }

    public IPage<User> query(UserQuery query) {
        return userDao.selectPage(query, query.build());
    }

    public List<User> list(UserQuery query) {
        return userDao.selectList(query.build());
    }

}
